{% load i18n form_tags %}

<h2>{% trans "Profile" %} <small>{% trans "Update your personal information" %}</small></h2>
<hr>
<form class="form-horizontal" method="POST" action="{% url 'core:user_profile' %}" 
      id="uprefs_form">
  {% csrf_token %}
  {% render_form form %}
  <div class="form-actions">
    <button type="submit" class="btn btn-primary btn-lg col-sm-offset-4 col-sm-4" id="update" name="update">{% trans 'Update' %}</button>
  </div>
</form>
