{% extends "registration/base.html" %}

{% load i18n form_tags %}

{% block main_content %}
  <h3 class="text-center">{% trans "Forgot password?" %}</h3>
  <form class="form-horizontal" method="post" style="margin-top: 20px">
    {% csrf_token %}
    {% render_field form.email %}
    <div class="col-sm-offset-4 col-sm-5">
      <button type="submit" class="btn btn-primary col-lg-12">{% trans "Submit" %}</button>
    </div>
  </form>
{% endblock %}
