// Import boostrap variables including default color palette and fonts
@import "../../node_modules/bootstrap/less/variables.less";

// Import datepicker component
@import "bootstrap-datetimepicker.less";
